--first import dem functions we use everywhere
local MachSkinCheck = MachAddon.MachSkinCheck
local speedCalc = MachAddon.speedCalc

--Ring attraction time
local ringRadius = { --keys are mach's filled bars, value is the ring attract radius
	[0] = 1, --just in case weird shit happens
	[1] = 75,
	[2] = 82,
	[3] = 90,
	[4] = 90
}
--this table holds the accepted object types to attract.
--No we're not adding ringslinger rings
local ringtable = {
	--This is just used for a quick check
	--also gonna use this to store conversions
	__index = false, --this is for objects that aren't rings
	[MT_RING] = MT_FLINGRING, --rings will be converted to flingrings
	[MT_FLINGRING] = true,
	[MT_COIN] = MT_FLINGCOIN, --coins will do the same
	[MT_FLINGCOIN] = true
}

--this function handles the ring detection logic
local function GetRing(pmo, ring)
	local ringtype = ring.type
	local isRingValid = ringtable[ring.type]
	local radius = ringRadius[pmo.player.machvars.barsFilled] * pmo.scale --get the proper range scaled with the player size
	local dist = FixedHypot(ring.x - pmo.x, FixedHypot(ring.y - pmo.y, ring.z - pmo.z))
	
	--don't collect the ring if it's too high up
	if not isRingValid or ring.mach_ringAttract then
		return
	elseif dist > radius then
		return
	end
	
	if isRingValid ~= true then --true is not equal to MT_FLINGRING, pretty cool huh?
		local x,y,z = ring.x, ring.y, ring.z
		P_RemoveMobj(ring)
		ring = P_SpawnMobj(x, y, z, isRingValid)
	end
	
	--ring.flags = $ & ~MF_SOLID
	--ring.flags = $ | MF_NOCLIP | MF_NOGRAVITY | MF_FLOAT | MF_NOCLIPHEIGHT | MF_SCENERY
	ring.flags = $ | MF_SCENERY | MF_NOCLIP | MF_NOCLIPHEIGHT
	
	ring.target = pmo
	ring.mach_ringAttract = true
	--P_MoveOrigin(ring, pmo.x, pmo.y, pmo.z)
end

--this function does the actual search for rings.
local function FindRings(p)
	if not (MachSkinCheck(p) and p.machvars.isBoosting) then
		return
	end
	
	--time to search!
	local pmo = p.mo
	
	--check if player is in the pain state
	if pmo.state == S_PLAY_PAIN then
		return --they are in pain, do not attract rings.
	end
	
	--get the radius value with player scale
	local radius = (ringRadius[pmo.player.machvars.barsFilled] + 2) * pmo.scale
	
	local x, y = pmo.x, pmo.y
	
	searchBlockmap("objects", GetRing, pmo, x - radius,
	x + radius, y - radius, y + radius)
end


--This function manages the ring movement when in attract mode
local function ringMovement(ring)
	if ring.mach_ringAttract then
		
		--get mach's mobj
		local pmo = ring.target
		local pscale = pmo.scale
		local rampUpSpeed = 4 * pscale
		
		--delete the ring if Mach fucking gets evicerated
		if not pmo then
			P_RemoveMobj(ring)
			return
		elseif not pmo.valid then
			P_RemoveMobj(ring)
			return
		--also delete it if the player is in pain
		elseif pmo.state == S_PLAY_PAIN then
			P_RemoveMobj(ring)
			return
		end
			
		if not ring.mach_speed then
			ring.mach_speed = FixedDiv(speedCalc(pmo), 3*pscale) - rampUpSpeed
		end
		
		ring.mach_speed = $ + rampUpSpeed --slowly ramp up speed each tic
		
		--if the rings are too fast, remove it and give mach the ring
		--this is a bandaid fix in case the ring goes so fast that it's
		--impossible to collect somehow
		if ring.mach_speed > 200*pscale then
			P_RemoveMobj(ring)
			P_GivePlayerRings(pmo.player, 1)
			return
		end
		
		A_HomingChase(ring, ring.mach_speed, 0)
	end
end


--hooks
addHook("PlayerThink", FindRings)
addHook("MobjThinker", ringMovement, MT_FLINGRING)
addHook("MobjThinker", ringMovement, MT_FLINGCOIN)
